/**
 * File Name: game.js
 * Bonus File Content URL: https://leanpub.com/LoRD
 * Description: monolithic File for controlling and displaying game scenes;
 *  managing global variables throughout game state.
 * Author: Stephen Gose
 * Version: 0.0.0.15
 * Author URL: https://www.stephen-gose.com/
 * Support: support@pbmcube.com
 *
 * Copyright  1974-2017 Stephen Gose LLC. All rights reserved. 
 * 
 * Do not sell! Do not distribute!
 * This is a licensed permission file. Please refer to Terms of Use
 *   and End Users License Agreement (EULA).
 * Redistribution of part or whole of this file and
 * the accompanying files is strictly prohibited.
 *
 */
 var GAMEAPP = GAMEAPP || {};
 var GAMEAPP = {
	 //US Copr. or Copyright
	Copr: "Copyright  1974-2016, Stephen Gose. All rights reserved.\n", 
	// Here we have some global level vars that persist regardless of State. 
	score: 0,
	clicks: 0,
	// If the music in your game, and it needs to play through-out a few State
	//	swaps, then you could reference it below. 
	//music: null,
	//Toggle background music theme on or off; starts in "on/true" state 
	//musicToggle: true,
	//Your game can check MYGAMEAPP.orientated in the game loops 
	//	to know if it should pause or not.
	//orientated: false
	
	//Grid Tile-Map configurations
	tileSize: 64, //twice the size of an avatar icon
	numRows: 3, //adjustable for your game
	numCols: 3, //adjustable for your game
	tileSpacing: 2,  //adjustable for your game
	tilesArray: [], //one way; thousand more to choose
	
	//methods
	_audioMgr: function(mode, game) {
		switch(mode) {
			case 'init': {
				GAMEAPP.Storage.initUnset('GAMEAPP.audio', true);
				GAMEAPP._audioStatus = GAMEAPP.Storage.get('GAMEAPP.audio');
				// GAMEAPP._soundClick = game.add.audio('audio-click');
				GAMEAPP._sound = [];
				GAMEAPP._sound['click'] = game.add.audio('audio-click');
				if(!GAMEAPP._soundMusic) {
					GAMEAPP._soundMusic = game.add.audio('audio-theme',1,true);
					GAMEAPP._soundMusic.volume = 0.5;
				}
				break;
			}
			case 'on': {
				GAMEAPP._audioStatus = true;
				break;
			}
			case 'off': {
				GAMEAPP._audioStatus = false;
				break;
			}
			case 'switch': {
				GAMEAPP._audioStatus =! GAMEAPP._audioStatus;
				break;
			}
			default: {}
		}
		if(GAMEAPP._audioStatus) {
			GAMEAPP._audioOffset = 0;
			if(GAMEAPP._soundMusic) {
				if(!GAMEAPP._soundMusic.isPlaying) {
					GAMEAPP._soundMusic.play('',0,1,true);
				}
			}
		}
		else {
			GAMEAPP._audioOffset = 4;
			if(GAMEAPP._soundMusic) {
				GAMEAPP._soundMusic.stop();
			}
		}
		GAMEAPP.Storage.set('GAMEAPP.audio',GAMEAPP._audioStatus);
		game.buttonAudio.setFrames(GAMEAPP._audioOffset+1, GAMEAPP._audioOffset+0, GAMEAPP._audioOffset+2);
	},
	_playAudio: function(sound) {
		if(GAMEAPP._audioStatus) {
			if(GAMEAPP._sound && GAMEAPP._sound[sound]) {
				GAMEAPP._sound[sound].play();
			}
		}
	}
};

//Create game namespace away from the window global
GAMEAPP.Boot = function (game) {
	console.log(" %c Starting my Game Prototype boot ", "color:white; background:red");
};

;//Closes any previous scripts
//(function() {

//Example: 1.2 Starting the Game.js begins
var gWidth = 800;	//Using Golden Ration is important.
var gHeight = 500;	//Using Golden Ration is important.
//Lessons learned from colleagues
	//initial size determined
	isMobile=navigator.userAgent.indexOf("Mobile");
	if (isMobile != -1) {
		//-1 is desktop/anything other than mobile device
		console.log("isMobile="+isMobile);
		gWidth = window.innerWidth * window.devicePixelRatio;
		gHeight = window.innerHeight * window.devicePixelRatio;
	}
var game = new Phaser.Game(gWidth, gHeight, Phaser.AUTO, "gContent");
//Example: 1.2 ends
//Example 1.3 to 1.19: Creating States in Game.js begins
//Step 3) new game state additions:
var mainState = {
	init: function(){
		//stuff to generate in this function
		console.log(" %c Starting my awesome Game Prototype | https://renown-games.com/shop | \u2665\u2665\u2665\u2665\u2665 ",
		"color:white; background:blue");
	},
	
	create: function() {
		//Example 1.6: Additional Phaser Properties begins
		console.log("mainState Ready!");
		//Set a neutral background color
		game.stage.backgroundColor = "#369";
		//Set game to ARCADE physics systemLanguage
		game.physics.startSystem(Phaser.Physics.ARCADE);
		game.renderer.renderSession.roundPixels = true;
		game.world.enableBody = true;
		//Example 1.6: ends
		//Example 1.7: Prototyping Graphics begins
		//create a character avatar 
		this.player =  
			game.add.sprite(32,32,
				box({length:32,width:32,color:'#00F'})); 
		this.cursor = game.input.keyboard.createCursorKeys();
		this.player.body.collideWorldBounds = true;
		//Example 1.7: ends
		//create an opponet  
		this.enemy = game.add.sprite(game.world.width/6 +64,32,
			box({length:32,width:32,color:'#0F0'}));
		this.enemy.body.collideWorldBounds = true;
		this.enemy.enableBody = true;
		this.enemy.body.immovable = true;
		game.physics.enable(this.enemy, Phaser.Physics.ARCADE);
		//Example 1.12: World Boundaries Integration begins
		//Create Room Walls on stage
		this.Room = game.add.group();  
		this.Room.enableBody = true; 
		game.physics.enable(this.Room, Phaser.Physics.ARCADE);
		
		var NorthWall = game.add.sprite(0,0,
			box({length:game.world.width,width:16,color:'#999'}));	
		NorthWall.enableBody = true;
		NorthWall.body.immovable = true;
		this.Room.add(NorthWall);
		
		var SouthWall = game.add.sprite(0,game.world.height-16,
			box({length:game.world.width,width:16,color:'#999'}));
		SouthWall.body.immovable = true;
		this.Room.add(SouthWall);
		//Example 1.12: ends
		//Example 1.13: World Boundaries Integration begins
		var WestWall = game.add.sprite(0,16,
			box({length:16,width:game.world.height-32,color:'#999'}));
		WestWall.body.immovable = true;    
		this.Room.add(WestWall);
		
		var EastWall = game.add.sprite(game.world.width-16,16,
			box({length:16,width:game.world.height-32,color:'#999'}));
		EastWall.body.immovable = true;    
		this.Room.add(EastWall);
		//Example 1.13: ends
		//Example 1.14: Interior Boundaries Integration begins
		var interior1Wall = game.add.sprite(game.world.width/6,16,
			box({length:16,width:game.world.width/4,color:'#AAA'}));
		interior1Wall.body.immovable = true;    
		this.Room.add(interior1Wall);
		
		var interior2Wall = game.add.sprite(16,game.world.height/2,
			box({length:game.world.width/2,width:16,color:'#AAA'}));
		interior2Wall.body.immovable = true;    
		this.Room.add(interior2Wall);
		//Example 1.14: ends
	},	//the comma is very important.
	
	update: function() {
		//Example 1.10: Prototyping Movement Properties
		//frame refresh and display updates
		var speed = 250;  
		this.player.body.velocity.x = 0;  
		this.player.body.velocity.y = 0;
		//Example 1.10: ends
		//Example 1.11: Movement Arrows Integration begins
		//monitor player's movement input
		if (this.cursor.up.isDown){  
			this.player.body.velocity.y -= speed;  
		}else if (this.cursor.down.isDown){  
			this.player.body.velocity.y += speed;  
		}else if (this.cursor.right.isDown){  
			this.player.body.velocity.x += speed;  
		}else if (this.cursor.left.isDown){  
			this.player.body.velocity.x -= speed;  
		} 
		//Example 1.11: ends
		game.physics.arcade.collide(this.player, this.Room);
		//Example 1.15: Collision Detection Integration
		game.physics.arcade.overlap(this.player,this.enemy,combatEncounter,null,this);  
		//Example 1.15: ends
	}
}; //the semi-colon is very important.
//Example 1.17: New Game Over State begins
var gameOverState = {  
	create: function(){ 
		//Example 1.18: Elementary HUD Creation begins
		label = game.add.text(game.world.width/2, game.world.height/2,"Game Over \n Press the SPACE bar to start again",{font: "22px Arial", fill: "#FFF", align:"center"});  
		label.anchor.setTo(0.5,0.5); 
		//Example 1.18: ends
		this.spacebar = this.game.input.keyboard.addKey(Phaser.Keyboard.SPACEBAR); 
	
	}, //comma very important here  
	update: function(){  
		//Example 1.19: Responding to User Input begins
		if(this.spacebar.isDown){  
			game.state.start('main');  
		}  
		//Example 1.19: ends
	}  
};  
//Example 1.17: ends
//Process conflict
var combat = {  
	create: function(){ 
		//Set a neutral background color
		console.log("Combat Ready!");
		game.stage.backgroundColor = "#300";
		//Set game to ARCADE physics systemLanguage
		game.physics.startSystem(Phaser.Physics.ARCADE);
		game.renderer.renderSession.roundPixels = true;
		game.world.enableBody = true;
		
		label = game.add.text(game.world.width/2, game.world.height/2,"Combat Encouter \n Press the SPACE bar to return",{font: "22px Arial", fill: "#FFF", align:"center"});  
		label.anchor.setTo(0.5,0.5); 
		
		this.spacebar = this.game.input.keyboard.addKey(Phaser.Keyboard.SPACEBAR); 
		
		//create a character avatar 
		this.player =  
		game.add.sprite(32,32,box({length:32,width:32,color:'#00F'})); 
		this.cursor = game.input.keyboard.createCursorKeys();
		this.player.body.collideWorldBounds = true;
		
		//create an opponet  
		this.enemy = game.add.sprite(game.world.width-64,game.world.height-64,box({length:32,width:32,color:'#0F0'}));
		this.enemy.body.collideWorldBounds = true;
		this.enemy.enableBody = true;
		this.enemy.body.immovable = true;
		game.physics.enable(this.enemy, Phaser.Physics.ARCADE);
			
		//Create Room Walls on stage
		this.Room = game.add.group();  
		this.Room.enableBody = true; 
		game.physics.enable(this.Room, Phaser.Physics.ARCADE);
		
		var NorthWall = game.add.sprite(0,0,box({length:game.world.width,width:16,color:'#999'}));	
		NorthWall.enableBody = true;
		NorthWall.body.immovable = true;
		this.Room.add(NorthWall);
		
		var SouthWall = game.add.sprite(0,game.world.height-16,box({length:game.world.width,width:16,color:'#999'}));
		SouthWall.body.immovable = true;
		this.Room.add(SouthWall);
		
		var WestWall = game.add.sprite(0,16,box({length:16,width:game.world.height-32,color:'#999'}));
		WestWall.body.immovable = true;    
		this.Room.add(WestWall);
		
		var EastWall = game.add.sprite(game.world.width-16,16,box({length:16,width:game.world.height-32,color:'#999'}));
		EastWall.body.immovable = true;    
		this.Room.add(EastWall);
		 
	
	}, //comma very important here  
	update: function(){  
		if(this.spacebar.isDown){  
			game.state.start('main');  
		}
		//frame refresh and display updates
		var speed = 250;  
		this.player.body.velocity.x = 0;  
		this.player.body.velocity.y = 0;

		//monitor player's movement input
		if (this.cursor.up.isDown){  
			this.player.body.velocity.y -= speed;  
		}
		if (this.cursor.down.isDown){  
			this.player.body.velocity.y += speed;  
		}
		if (this.cursor.right.isDown){  
			this.player.body.velocity.x += speed;  
		}
		if (this.cursor.left.isDown){  
			this.player.body.velocity.x -= speed;  
		} 

		game.physics.arcade.collide(this.player, this.Room);
		
		game.physics.arcade.overlap(this.player,this.enemy,combatEncounter,null,this);  		
	}  
}; 
//Example 1.16: Collision Results Determination begins
//character's death  
var handlePlayerDeath = function(player,enemy){
	//kill off the avatar
	player.kill();
	//change to Game Over scene
	game.state.start("gameOver"); 
} 
//Example 1.16: ends
//character's initiates combat  
var combatEncounter = function(player,enemy){  
	//change to Game Over scene
	game.state.start("combat"); 
}
//Example 1.7: Prototyping Graphics begins
//create a box Image (pseudo graphics) for the HTML5 canvas. 
var box = function(options) {  
	var bxImg = this.game.add.bitmapData(options.length,options.width);  
	bxImg.ctx.beginPath();  
	bxImg.ctx.rect(0,0,options.length,options.width);  
	bxImg.ctx.fillStyle = options.color;  
	bxImg.ctx.fill();  
	return bxImg;  
};  

//Step 1) Lets tell Phaser about our new scenes
//Phaser uses our code and gives it a name of main.
game.state.add("main", mainState);
game.state.add("gameOver", gameOverState); 
game.state.add("combat", combat);   
//tells Phase to start using it.
game.state.start("main"); //tells Phase to start using it.
//Example 1.7: ends
//Optimized method; see chapter on optimization.

//})(); //immediate execution; Self-Invoking Function   
