	/**
	 * File Name: boot.js
	 * Live Demo: https://makingbrowsergames.com/starterkits/
	 * Source Code: https://makingbrowsergames.com/starterkits/
	 * Description: File for controlling initial game shell launch; managing global variables throughout game state.
	 * Author: Stephen Gose
	 * Version: 0.0.0.xxxx semver launch date
	 * Author URL: https://www.stephen-gose.com/
	 * Support: support@pbmcube.com
	 *
	 * \u00A9 Copyright ©  1974-2017 Stephen Gose LLC. All rights reserved. 
	 * 
	 * Do not sell! Do not distribute!
	 * This is a licensed permission file. Please refer to Terms of Use
	 *   and End Users License Agreement (EULA).
	 * Search for [ //**TODO** ] to tailor this file for your own use 
	 *   and will void any support agreement.
	 *
	 * Redistribution of part or whole of this file and
	 * the accompanying files is strictly prohibited.
	 *
	 */

	"use strict";
	window.GAMEAPP.state.boot = {
		preload: function(){
			console.log(" %c  Game Prototype boot configure browser settings  ", "color:white; background:red");
			
			//Debug Plugin - for tutorials, development and books
			//this.game.add.plugin(Phaser.Plugin.Debug);
// -------------------------------------------
// TODO: Last chance to start WebSocket automatically!!
// -------------------------------------------			
//
// =================================================			
// CORS:
// - https://developer.mozilla.org/en-US/docs/Web/HTTP/Access_control_CORS
// - https://enable-cors.org/
// Avoid this anti-pattern of omitting the protocols schemes in script "src";
// Cross origin requests are only supported for protocol schemes: 
//		http, data, chrome, chrome-extension, https.
			this.load.crossOrigin = 'anonymous';
//
// =================================================
			// required assets for following states here	
			//<SKU> mainMenu & Story collapse into one scene
			this.game.load.path = "assets/images/staticScenes/";			
			this.game.load.image('background', 'titleScene.jpg');	
			this.game.load.image('creditScene', 'creditsScene.jpg');	
			this.game.load.image('langScene', 'languageScene.jpg');
			this.game.load.image('mainMenu', 'mainMenu.jpg');
			//this.game.load.image('menus', 'MenuPanelHUD2.jpg');
			this.game.load.image('storyScene', 'storyScene.jpg');
			this.game.load.image('titleScene', 'titleScene.jpg');

			//Language Scene uses StoryPanelHud
			this.game.load.path = "assets/images/flags/";
			this.load.spritesheet('chinaBtn', 'china.png', 50, 50);
			this.load.spritesheet('dmarkBtn', 'denmark.png', 50, 50);
			this.load.spritesheet('englishBtn', 'english.png', 50, 50);
			this.load.spritesheet('frenchBtn', 'france.png', 50, 50);
			this.load.spritesheet('germanBtn', 'germany.png', 50, 50);
			this.load.spritesheet('italyBtn', 'italy.png', 50, 50);
			this.load.spritesheet('japanBtn', 'japan.png', 50, 50);
			this.load.spritesheet('nlandBtn', 'netherlands.png', 50, 50);
			this.load.spritesheet('portugalBtn', 'portugal.png', 50, 50);
			this.load.spritesheet('spainBtn', 'spain.png', 50, 50);
			this.load.spritesheet('swedenBtn', 'sweden.png', 50, 50);
			
			//UI Buttons
			this.game.load.path = "";
			this.load.spritesheet('button', 'assets/spriteSheets/mmog-sprites-silver.png', 129, 30);
			this.game.load.atlas('button-continue','assets/spriteSheets/RightArrow-Phaser.png','assets/spriteSheets/arrowR-sprites.json');
			this.game.load.atlas('button','assets/spriteSheets/mmog-sprites-silver.png','assets/spriteSheets/mmog-sprites.json');
			this.game.load.atlas('langButton','assets/spriteSheets/language.png','assets/spriteSheets/language.json');
			
			//Load button for twitter
			this.game.load.image('twitter','assets/images/twitter.png');
			
			//Game Mechanics (Space & Area of Interest); set world size
			this.game.world.setBounds(0, 0, window.GAMEAPP.worldWidth, window.GAMEAPP.worldHeight);
			//Game Mechanics (Space & Area of Interest); set world size
			this.enableScaling();
					
		},
		create: function(){
			//this.background = this.add.image(0, 0, 'background');
			
			// add all game states
			for(var stateName in window.GAMEAPP.state){
				this.game.state.add(stateName, window.GAMEAPP.state[stateName]);
			}
			
			//ACTIVATE FOR ITCH.IO BUILDS
			//local storage capacity = 5MB per
			//The localStorage object stores the data with no expiration date. 
			//The data will not be deleted when the browser is closed,
			//   and will be available whenever.
			if(localStorage.firstRunDate !== "undefined"){
				localStorage.clear();
				sessionStorage.clear();
			}else{
				console.log(" %c  Resorting to Cookie storage.   ","color:white; background:red");
			}
			
			//
			if(localStorage.firstRun != "DONE"){
				console.log(" %c  Setting up for first-time run  ","color:white; background:red");
				firstRun();
			}
			// Game Mechanics (Attributes, Objects & Roses);
			// goto load state
			this.game.state.start("load");
		},
		//Game Mechanics (Space & Area of Interest); set world size
		enableScaling: function(){
			var game = this.game;
			game.scale.parentIsWindow = (game.canvas.parentNode == document.body);
			game.scale.scaleMode = Phaser.ScaleManager.SHOW_ALL;
		}
	};

// =================================================
// -------------------------------------------
// Supporting game Function & Classes
// -------------------------------------------
// =================================================
	//local and session storage management
function firstRun(){
	var d = new Date();
	localStorage.clear();
	initScoreSystem();
	localStorage.setItem("curBg", "0");
	localStorage.setItem("firstRun", "DONE");
	localStorage.setItem("firstRunDate", d.getTime());
};

//
// =================================================
function initScoreSystem(){
	//Leaderboard
	resetLeaderboard();

	//sounds
	localStorage.setItem("musicVolume", "1.5");
	localStorage.setItem("fxVolume", "1.5");
	localStorage.setItem("musicTrack", "yewtree");

	//init keys
	var keys = getDefaultUserKeys();
	saveUserKeys(keys);

	//init language
	localStorage.setItem("curLang", "EN");
};

//
// =================================================
function resetLeaderboard(){
	for(var i = 0; i < 5; i++){
		localStorage.setItem("leaderboard_" + i, 1000 - (i * 100));
	}
	//prime the local database; syn with remote using PouchDB
	localStorage.setItem("leaderNames_0", "Charles");
	localStorage.setItem("leaderNames_1", "Dave");
	localStorage.setItem("leaderNames_2", "Richard");
	localStorage.setItem("leaderNames_3", "Zoe");
	localStorage.setItem("leaderNames_4", "Paul");
};

//
// =================================================
// -------------------------------------------
// Keyboard management
// -------------------------------------------
// =================================================
function getKeyName(keyNum){
	switch(keyNum){
		case Phaser.Keyboard.A:
		return "A";
		break;
		case Phaser.Keyboard.B:
		return "B";
		break;
		case Phaser.Keyboard.C:
		return "C";
		break;
		case Phaser.Keyboard.D:
		return "D";
		break;
		case Phaser.Keyboard.E :
		return "E ";
		break;
		case Phaser.Keyboard.F:
		return "F";
		break;
		case Phaser.Keyboard.G:
		return "G";
		break;
		case Phaser.Keyboard.H:
		return "H";
		break;
		case Phaser.Keyboard.I:
		return "I";
		break;
		case Phaser.Keyboard.J:
		return "J";
		break;
		case Phaser.Keyboard.K:
		return "K";
		break;
		case Phaser.Keyboard.L:
		return "L";
		break;
		case Phaser.Keyboard.M:
		return "M";
		break;
		case Phaser.Keyboard.N:
		return "N";
		break;
		case Phaser.Keyboard.O:
		return "O";
		break;
		case Phaser.Keyboard.P:
		return "P";
		break;
		case Phaser.Keyboard.Q:
		return "Q";
		break;
		case Phaser.Keyboard.R:
		return "R";
		break;
		case Phaser.Keyboard.S:
		return "S";
		break;
		case Phaser.Keyboard.T:
		return "T";
		break;
		case Phaser.Keyboard.U:
		return "U";
		break;
		case Phaser.Keyboard.V:
		return "V";
		break;
		case Phaser.Keyboard.W:
		return "W";
		break;
		case Phaser.Keyboard.X :
		return "X ";
		break;
		case Phaser.Keyboard.Y :
		return "Y ";
		break;
		case Phaser.Keyboard.Z :
		return "Z ";
		break;
	//Numbers row
		case Phaser.Keyboard.ZERO:
		return "0";
		break;
		case Phaser.Keyboard.ONE:
		return "1";
		break;
		case Phaser.Keyboard.TWO:
		return "2";
		break;
		case Phaser.Keyboard.THREE:
		return "3";
		break;
		case Phaser.Keyboard.FOUR:
		return "4";
		break;
		case Phaser.Keyboard.FIVE:
		return "5";
		break;
		case Phaser.Keyboard.SIX:
		return "6";
		break;
		case Phaser.Keyboard.SEVEN:
		return "7";
		break;
		case Phaser.Keyboard.EIGHT:
		return "8";
		break;
		case Phaser.Keyboard.NINE:
		return "9";
		break;
	//Numbers on 10-key pad
		case Phaser.Keyboard.NUMPAD_0:
		return "NUMPAD_0";
		break;
		case Phaser.Keyboard.NUMPAD_1:
		return "NUMPAD_1";
		break;
		case Phaser.Keyboard.NUMPAD_2:
		return "NUMPAD_2";
		break;
		case Phaser.Keyboard.NUMPAD_3:
		return "NUMPAD_3";
		break;
		case Phaser.Keyboard.NUMPAD_4:
		return "NUMPAD_4";
		break;
		case Phaser.Keyboard.NUMPAD_5:
		return "NUMPAD_5";
		break;
		case Phaser.Keyboard.NUMPAD_6:
		return "NUMPAD_6";
		break;
		case Phaser.Keyboard.NUMPAD_7:
		return "NUMPAD_7";
		break;
		case Phaser.Keyboard.NUMPAD_8:
		return "NUMPAD_8";
		break;
		case Phaser.Keyboard.NUMPAD_9:
		return "NUMPAD_9";
		break;
	
		case Phaser.Keyboard.NUMPAD_ADD:
		return "NUMPAD_ADD";
		break;
		case Phaser.Keyboard.NUMPAD_DECIMAL:
		return "NUMPAD_DECIMAL";
		break;
		case Phaser.Keyboard.NUMPAD_DIVIDE:
		return "NUMPAD_DIVIDE";
		break;
		case Phaser.Keyboard.NUMPAD_ENTER:
		return "NUMPAD_ENTER";
		break;
		case Phaser.Keyboard.NUMPAD_MULTIPLY:
		return "NUMPAD_MULTIPLY";
		break;
		case Phaser.Keyboard.NUMPAD_SUBTRACT:
		return "NUMPAD_SUBTRACT";
		break;
	//Function Keys
		case Phaser.Keyboard.F1:
		return "F1";
		break;
		case Phaser.Keyboard.F2:
		return "F2";
		break;
		case Phaser.Keyboard.F3:
		return "F3";
		break;
		case Phaser.Keyboard.F4:
		return "F4";
		break;
		case Phaser.Keyboard.F5:
		return "F5";
		break;
		case Phaser.Keyboard.F6:
		return "F6";
		break;
		case Phaser.Keyboard.F7:
		return "F7";
		break;
		case Phaser.Keyboard.F8:
		return "F8";
		break;
		case Phaser.Keyboard.F9:
		return "F9";
		break;
		case Phaser.Keyboard.F10:
		return "F10";
		break;
		case Phaser.Keyboard.F11:
		return "F11";
		break;
		case Phaser.Keyboard.F12:
		return "F12";
		break;
		case Phaser.Keyboard.F13:
		return "F13";
		break;
		case Phaser.Keyboard.F14:
		return "F14";
		break;
		case Phaser.Keyboard.F15:
		return "F15";
		break;
	//Utility keys
		case Phaser.Keyboard.SPACEBAR:
		return "SPACEBAR";
		break;
		case Phaser.Keyboard.ENTER:
		return "ENTER";
		break;
		case Phaser.Keyboard.TAB:
		return "TAB";
		break;
		case Phaser.Keyboard.ESC:
		return "ESC";
		break;
	//arrow keys assigned
		case Phaser.Keyboard.UP:
		return "UP";
		break;
		case Phaser.Keyboard.DOWN:
		return "DOWN";
		break;
		case Phaser.Keyboard.LEFT:
		return "LEFT";
		break;
		case Phaser.Keyboard.RIGHT:
		return "RIGHT";
		break;

		case Phaser.Keyboard.ALT:
		return "ALT";
		break;
		case Phaser.Keyboard.BACKSPACE :
		return "BACKSPACE ";
		break;
		case Phaser.Keyboard.BACKWARD_SLASH:
		return "BACKWARD_SLASH";
		break;
		case Phaser.Keyboard.CAPS_LOCK:
		return "CAPS_LOCK";
		break;
		case Phaser.Keyboard.CLEAR:
		return "CLEAR";
		break;
		case Phaser.Keyboard.CLOSED_BRACKET:
		return "CLOSED_BRACKET";
		break;
		case Phaser.Keyboard.COLON:
		return "COLON";
		break;
		case Phaser.Keyboard.CONTROL:
		return "CONTROL";
		break;
		case Phaser.Keyboard.DELETE:
		return "DELETE";
		break;
		case Phaser.Keyboard.END:
		return "END";
		break;
		case Phaser.Keyboard.EQUALS:
		return "EQUALS";
		break;
		case Phaser.Keyboard.HELP:
		return "HELP";
		break;
		case Phaser.Keyboard.HOME:
		return "HOME";
		break;
		case Phaser.Keyboard.INSERT:
		return "INSERT";
		break;
		case Phaser.Keyboard.MINUS:
		return "MINUS";
		break;
		case Phaser.Keyboard.NUM_LOCK :
		return "NUM_LOCK ";
		break;
		case Phaser.Keyboard.OPEN_BRACKET:
		return "OPEN_BRACKET";
		break;
		case Phaser.Keyboard.PAGE_DOWN:
		return "PAGE_DOWN";
		break;
		case Phaser.Keyboard.PAGE_UP:
		return "PAGE_UP";
		break;
		case Phaser.Keyboard.PLUS:
		return "PLUS";
		break;
		case Phaser.Keyboard.QUESTION_MARK:
		return "QUESTION_MARK";
		break;
		case Phaser.Keyboard.QUOTES :
		return "QUOTES ";
		break;
		case Phaser.Keyboard.SHIFT:
		return "SHIFT";
		break;
		case Phaser.Keyboard.TILDE :
		return "TILDE ";
		break;
		case Phaser.Keyboard.UNDERSCORE:
		return "UNDERSCORE";
		break;
		default:
		return "UNKNOWN";
		break;

	}
};

//
// =================================================
//retrieve keyboard configurations
function loadUserKeys(){
	var keys = [];
	keys.push(parseInt(localStorage.getItem("key_left")));
	keys.push(parseInt(localStorage.getItem("key_right")));
	keys.push(parseInt(localStorage.getItem("key_rotate_CW")));
	keys.push(parseInt(localStorage.getItem("key_rotate_CCW")));
	keys.push(parseInt(localStorage.getItem("key_soft_drop")));
	keys.push(parseInt(localStorage.getItem("key_hard_drop")));
	keys.push(parseInt(localStorage.getItem("key_hold")));
	keys.push(parseInt(localStorage.getItem("key_confirm")));
	keys.push(parseInt(localStorage.getItem("key_cancel")));
	return keys;
};

//
// =================================================
// save new keyboard configuration for non-US layout
function saveUserKeys(newValues){
	var i = 0;
	
	if(newValues.length != 9){
		return false;
	}
	//store user keyboard selections in localStorage for later.
	localStorage.setItem ("key_left", newValues[i]);
	i++;
	localStorage.setItem ("key_right", newValues[i]);
	i++;
	localStorage.setItem ("key_rotate_CW", newValues[i]);
	i++;
	localStorage.setItem ("key_rotate_CCW", newValues[i]);
	i++;
	localStorage.setItem ("key_soft_drop", newValues[i]);
	i++;
	localStorage.setItem ("key_hard_drop", newValues[i]);
	i++;
	localStorage.setItem ("key_hold", newValues[i]);
	i++;
	localStorage.setItem ("key_confirm", newValues[i]);
	i++;
	localStorage.setItem ("key_cancel", newValues[i]);
	console.log("Keyboard configurations saved");	//debug
	return true;
};

//
// =================================================
//default US keyboard assumed
function getDefaultUserKeys(){
	var keys = [
		Phaser.Keyboard.LEFT,
		Phaser.Keyboard.RIGHT,
		Phaser.Keyboard.UP,
		Phaser.Keyboard.Z,
		Phaser.Keyboard.DOWN,
		Phaser.Keyboard.SPACEBAR,
		Phaser.Keyboard.X,
		Phaser.Keyboard.ENTER,
		Phaser.Keyboard.ESC
	];
	return keys;
};

//
// =================================================
/* End of file */
/* Location: ./js/state/boot.js */