/**
 * File Name: menu.js
 * Live Demo: https://makingbrowsergames.com/starterkits/
 * Source Code: https://makingbrowsergames.com/starterkits/
 * Description: File for controlling and displaying game scenes; 
 *      managing global variables throughout game state in the gaming shell.
 * Author: Stephen Gose
 * Version: 0.0.0.xxxx semver launch date
 * Author URL: https://www.stephen-gose.com/
 * Support: support@pbmcube.com
 *
 * \u00A9 Copyright ©  1974-2017 Stephen Gose LLC. All rights reserved. 
 * 
 * Do not sell! Do not distribute!
 * This is a licensed permission file. Please refer to Terms of Use
 *   and End Users License Agreement (EULA).
 * Search for [ //**TODO** ] to tailor this file for your own use 
 *   and will void any support agreement.
 *
 * Redistribution of part or whole of this file and
 * the accompanying files is strictly prohibited.
 *
 * This file is automatically loaded from state/load.js
 * to change default state - change state/load.js at line: 34
 */

"use strict";

// 
// =================================================
window.GAMEAPP.state.menu = {
	preload: function(){
		console.log(" %c <SKU> rv_xxx Story & Main Menu Game Prototype", "color:white; background:blue");
		GAMEAPP.InfoText = "Main Menu Selections";
		
		//<SKU> mainMenu & Story collapse into one scene
		this.load.spritesheet('button', 'assets/spriteSheets/mmog-sprites-silver.png', 129, 30);
		GAMEAPP._assignLanguage(GAMEAPP.gameLanguage);
		//using altas for graphics; loading graphics for the main menu state now; 
		//	they should be in the cache when needed.
		//	game theme music file should be deferred to splash/language phase.
		//navigation and menu buttons; 
		//two methods to load spriteSheets: 1) classic or 2) texture atlas
		//load navigation buttons using classic method
		//using programmatic method
		//this._preloadResources();
		
		//new menu optimization from Mozart Music match game
		this.menuBtn = [];	//main menu button array
	},
	
	create: function(){
		console.log("starting story line & Main Menu state");
		this.game.add.image(0, 0, 'mainMenu');
		var style = {};
		//Static english language assignment here; use global variable to attach selected language
		var playtxt = this.add.text(0, 0, "Play" , GAMEAPP.styleBTN);	// "Play";
		var opttxt = this.add.text(0, 0, "Options", GAMEAPP.styleBTN); 	//"Game Options";
		var sharetxt =  this.add.text(0, 0, "Share", GAMEAPP.styleBTN); //"Share with a Friend";
		var moretxt = this.add.text(0, 0, "More", GAMEAPP.styleBTN); 	//"More Games";
		var credittxt = this.add.text(0, 0, "Credits", GAMEAPP.styleBTN);//"Credits";
		var webtxt = this.add.text(0, 0, "License", GAMEAPP.styleBTN);	//"WebMasters" && customer license;
		var donationtxt = this.add.text(0, 0, "Donations", GAMEAPP.styleBTN);//Crowd funding
		
		if (GAMEAPP.gameLanguage == 0) {
			this.game.add.image(0, 0, 'titleScene');
			style = { font: "12px Arial", fill: "#F90", align: "center", boundsAlignH: "left", boundsAlignV: "top", wordWrap: true, wordWrapWidth: 245 };
			this.introParatxt = this.add.text(1000,0, GAMEAPP.introPara, style);
			this.introParatxt.setShadow(3, 3, 'rgba(0,0,0,0.5)', 5);
			//this.introParatxt.setTextBounds(555, 70, 780, 400);
			this.introParatxt.fontWeight = 'bold';
		}else{
			this.game.add.image(0, 0, 'titleScene');
			// the alignment of the text is independent of the bounds, try changing to 'center' or 'right'
			style = { font: "14px Arial", fill: "#FFF", align: "center", boundsAlignH: "left", boundsAlignV: "top", wordWrap: true, wordWrapWidth: 230 };
			this.introParatxt = this.add.text(0,0, GAMEAPP.introPara, style);
			this.introParatxt.setShadow(3, 3, 'rgba(0,0,0,0.5)', 5);
			this.introParatxt.setTextBounds(550, 138, 780, 400);
			this.introParatxt.fontWeight = 'bold';
		}
		
		//Main Menu creation
		//this.soundButton = this.add.button(10, 10, 'soundButton', this.toggleThemeMusic, this);
		
		//new menu optimized
		/**
		//Play
		this.menuBtn[0] = this.add.button(670, 280, 'button', this.startGame, this, 2, 1, 0, 1);
		//Credit
		this.menuBtn[1] = this.add.button(670, 280, 'button', this.startGame, this, 2, 1, 0, 1);
		// Share - external
		this.menuBtn[2] = this.add.button(670, 280, 'button', this.startGame, this, 2, 1, 0, 1);
		// Game Options : Language Selection
		this.menuBtn[3] = this.add.button(670, 280, 'button', this.startGame, this, 2, 1, 0, 1);
		// More Games - external 
		this.menuBtn[4] = this.add.button(670, 280, 'button', this.startGame, this, 2, 1, 0, 1);
		// Web Masters & customer License - external
		this.menuBtn[5] = this.add.button(670, 280, 'button', this.startGame, this, 2, 1, 0, 1);
		// Donation Link - external
		this.menuBtn[6] = this.add.button(670, 280, 'button', this.startGame, this, 2, 1, 0, 1);
		
		//take each button sprite and create addition parameters
        this.menuBtn.forEach( function (key, index) {
			key.name = "menuBtn"+index;
			//key.alpha = .2;
			key.enableBody = true;
			key.inputEnabled = true;
            key.events.onInputDown.add( this.selectKeys, this);
            key.events..onInputOver.add(this.btnOver, this);
            key.customParams = {id: index}; 
			console.log("Created Key gfx: "+index);	//debug
        }, this);
		
		*/
	/**
	//TODO: integration to above optimized menu
	_labelButton: function(game,x,y,key,label,callback,  
		callbackContext, overFrame, outFrame, downFrame, upFrame){
		Phaser.Button.call(this,game,x,y,key,callback,  
		callbackContext, overFrame, outFrame, downFrame, upFrame);  
		//Style how you wish...    
		this.style = {'font': '10px Arial','fill':'black'};  
		this.anchor.setTo( 0.5, 0.5 );  
		this.label = new Phaser.Text(game, 0, 0, label, this.style);  
		//puts the label in the center of the button  
		this.label.anchor.setTo( 0.5, 0.5 );  
		this.addChild(this.label);  
		this.setLabel( label );  
		//adds button to game  
		game.add.existing( this );  	
	}
	_labelButton.prototype = Object.create(Phaser.Button.prototype);  
	_labelButton.prototype.constructor = _labelButton;  
	_labelButton.prototype.setLabel = function( label ) {  
		this.label.setText(label);  
	}
	*/
	// Game Mechanics (Attributes, Objects & Roses): UI;
		// Play 
		this.playButton = this.add.button(670, 280, 'button', this.startGame, this, 2, 1, 0, 1); // button frames: over, off, down
		this.playButton.anchor.set(0.5,0);
		this.playButton.scale.setTo(0.7,0.7);
		this.playButton.name = "play";
		this.playButton.onInputOver.add(this.btnOver, this);
		this.playButton.addChild(playtxt).anchor.set(0.5,0);
		
		// Credit 
		this.creditButton = this.add.button(615, 320, 'button', this.GameCredits, this, 2, 1, 0, 1); // button frames: over, off, down
		this.creditButton.anchor.set(0.5,0);
		this.creditButton.scale.setTo(0.7,0.7);
		this.creditButton.name = "credit";
		this.creditButton.onInputOver.add(this.btnOver, this);
		this.creditButton.addChild(credittxt).anchor.set(0.5,0);
		
		// Share - external
		this.shareButton = this.add.button(615, 360, 'button', this.GameShare, this, 2, 1, 0, 1); // button frames: over, off, down
		this.shareButton.anchor.set(0.5,0);
		this.shareButton.scale.setTo(0.7,0.7);
		this.shareButton.name = "share";
		this.shareButton.onInputOver.add(this.btnOver, this);
		this.shareButton.addChild(sharetxt).anchor.set(0.5,0);
		
		// Game Options : Language Selection
		this.optButton = this.add.button(615, 400, 'button', this.GameOptions, this, 2, 1, 0, 1); // button frames: over, off, down
		this.optButton.anchor.set(0.5,0);
		this.optButton.scale.setTo(0.7,0.7);
		this.optButton.name = "options";
		this.optButton.onInputOver.add(this.btnOver, this);
		this.optButton.addChild(opttxt).anchor.set(0.5,0);
		
		// More Games - external 
		this.moreButton = this.add.button(735, 320, 'button', this.MoreGame, this, 2, 1, 0, 1); // button frames: over, off, down
		this.moreButton.anchor.set(0.5,0);
		this.moreButton.scale.setTo(0.7,0.7);
		this.moreButton.name = "more";
		this.moreButton.onInputOver.add(this.btnOver, this);
		this.moreButton.addChild(moretxt).anchor.set(0.5,0);
		
		// Web Masters & customer License - external 
		this.webButton = this.add.button(735, 360, 'button', this.WebMaster, this, 2, 1, 0, 1); // button frames: over, off, down
		this.webButton.anchor.set(0.5,0);
		this.webButton.scale.setTo(0.7,0.7);
		this.webButton.name = "web";
		this.webButton.onInputOver.add(this.btnOver, this);
		this.webButton.addChild(webtxt).anchor.set(0.5,0);

		// Donation Link - external
		this.donaButton = this.add.button(735, 400, 'button', this.GameDonations, this, 2, 1, 0, 1); // button frames: over, off, down
		this.donaButton.anchor.set(0.5,0);
		this.donaButton.scale.setTo(0.7,0.7);
		this.donaButton.name = "dona";
		this.donaButton.onInputOver.add(this.btnOver, this);
		this.donaButton.addChild(donationtxt).anchor.set(0.5,0);

		this._toolTip = this.game.add.text(670, this.world.centerY-10, GAMEAPP.InfoText, style);
		this._toolTip.anchor.set(0.5,0.5);
		this._toolTip.setShadow(3, 3, 'rgba(0,0,0,0.5)', 5);
		this._toolTip.fontWeight = 'bold';
		// Game Mechanics (Space & Area of Interest): camera;
		this.camera.flash(0x000000, 500, false);
		
	},
	
	update: function(){
		
	},
	
	// =================================================
	// -------------------------------------------
	// Supporting game Function & Classes
	// -------------------------------------------
	// =================================================
	//
	// =================================================
	clickContinue: function() {
		//GAMEAPP._playAudio('click');
		// Game Mechanics (Space & Area of Interest): camera;
		this.camera.fade(0x000000, 200, false);
		this.time.events.add(200, function() {
			//this.game.state.start('Main');	//BBS & Flash website: TIGStart method
			//this.game.state.start('R6');		//ARRA Main Entrance rv_3 through rv_8
		}, this);
		
	},
	
	//
	// =================================================
	// Menu Button function
	// =================================================
	btnOver: function(butn){
		switch(butn.name){
			case "play":
				console.log("Play selected");
				this._toolTip.setText(GAMEAPP.playtxt); 
			break;
			case "credit":
				console.log("Credit selected");
				this._toolTip.setText(GAMEAPP.credittxt);
			break;
			case "share":
				console.log("Share this Game!");
				this._toolTip.setText(GAMEAPP.sharetxt);
			break;
			case "options":
				console.log("Options selected");
				this._toolTip.setText(GAMEAPP.opttxt);
			break;
			case "more":
				console.log("More selected");
				this._toolTip.setText(GAMEAPP.moretxt);
			break;
			case "web":
				console.log("Web selected");
				this._toolTip.setText("Webmasters");
			break;
			case "dona":
				console.log("Donations selected");
				this._toolTip.setText("Support us");
			break;
		}
	},
	// =================================================
	// Game Mechanics (Attributes, Objects & Roses);
	//   navigate to selected state
	// =================================================
	startGame: function (pointer) {
		
		//Ok, let's play the game!
		this.state.start('play');

	},
	
	GameCredits: function (pointer) {

		//Ok, switch to the credits and acknowledgements menu.
		this.state.start('credits');

	},
	
	GameOptions: function (pointer) {

		//Ok, switch to the language menu.
		this.state.start('language');

	},
	
	GameShare: function (pointer) {
		//Popup twitter window 
		var game_url = 'https://makingbrowsergames.com/starterkits/';
		var twitter_name = 'PBMCube';
		var tags = ['Phaser,phaser3'];
		
		//Ok, gamer want to share with friends; let's use twitter.
		window.open('https://twiter.com/share?text=I+am+creating+and+playing+fun+games+from+&via='+twitter_name+'&url='+game_url+'&hashtags='+tags.join(','), '_blank');

	},
	
	MoreGame: function (pointer) {

		//Ok, let's send them to our game Showcase.
		window.open("https://www.renown-games.com./index.html", "_blank");

	},
	
	WebMaster: function (pointer) {

		//Ok, send them our store to get a license 
		window.open("https://renown-games.com/shop/downloads/category/software-source-code/ssc5/", "_blank");

	},
	
	GameDonations: function (pointer) {

		//Ok, let's take their money and thank them.
		//  OR, we could send them to our Patron/KickStarter sites.
		window.open("https://www.paypal.me/pbmcube", "_blank");

	},

	
};

//
// =================================================
/* End of file */
/* Location: ./js/state/menu.js */